from dataclasses import dataclass
from src.config.config import Configuration
from src.rendering import GENERIC_FILE_MAPPING
from src.rendering.markdown import render_markdown, read_raw_markdown, rendered_markdown_to_plain_text
from enum import Enum

from PIL import Image
from datetime import datetime
import frontmatter


@dataclass
class ImageMetadata:
    width: int
    height: int
    alt: str
    exif: dict

@dataclass
class MarkdownMetadata:
    fontmatter: dict
    content: str
    preview: str


@dataclass
class FileMetadata:
    typeMeta: MarkdownMetadata | None


@dataclass
class TemplateFile:
    name: str
    path: str
    proper_name: str
    extension: str
    categories: list[str]
    date_modified: str
    date_created: str
    size_kb: int
    metadata: ImageMetadata | FileMetadata | None
    dir_item_count: int
    is_dir: bool


def format_date(timestamp):
    return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d")


class TemplateHelpers:
    def __init__(self, config: Configuration):
        self.config: Configuration = config


    def build_metadata_for_file(self, path: str, categories: list[str] = []):
        """Builds metadata for a file"""
        file_path = self.config.content_dir / path
        for k in categories:
            if k == "image":
                img = Image.open(file_path)
                exif = img._getexif()
                orientation = exif.get(274, 1) if exif else 1
                width, height = img.width, img.height
                if orientation in [5, 6, 7, 8]:
                    width, height = height, width
                return ImageMetadata(
                    width=width,
                    height=height,
                    alt=file_path.name,
                    exif=img.info,
                )
            elif k == "document":
                ret = None
                with open(file_path, "r") as fdoc:
                    ret = FileMetadata(None)
                if file_path.suffix[1:].lower() == "md":
                    ret.typeMeta = MarkdownMetadata({}, "", "")
                    ret.typeMeta.fontmatter = frontmatter.load(file_path)
                    ret.typeMeta.content = render_markdown(file_path)
                    ret.typeMeta.rawContent = read_raw_markdown(file_path)
                    ret.typeMeta.rawText = rendered_markdown_to_plain_text(ret.typeMeta.content)
                    ret.typeMeta.preview = ret.typeMeta.rawText[:500] + "..."
                return ret
        return None


    def get_folder_contents(self, path: str = ""):
        """Returns the contents of a folder as a list of TemplateFile objects

        The metadata field is populated with the appropriate metadata object
        """
        search_contnet_path = self.config.content_dir / path
        files = search_contnet_path.glob("*")
        ret = []
        for f in files:
            t = TemplateFile(
                name=f.name,
                path=str(f.relative_to(self.config.content_dir)),
                proper_name=f.stem,
                extension=f.suffix.lower(),
                categories=[],
                date_modified=format_date(f.stat().st_mtime),
                date_created=format_date(f.stat().st_ctime),
                size_kb=f.stat().st_size / 1024,
                metadata=None,
                dir_item_count=len(list(f.glob("*"))) if f.is_dir() else 0,
                is_dir=f.is_dir(),
            )
            if f.is_file():
                for k, v in GENERIC_FILE_MAPPING.items():
                    if f.suffix[1:].lower() in v:
                        t.categories.append(k)
            t.metadata = self.build_metadata_for_file(f, t.categories)
            ret.append(t)
        return ret

    def get_sibling_content_files(self, path: str = ""):
        search_contnet_path = self.config.content_dir / path
        files = search_contnet_path.glob("*")
        return [
            (file.name, str(file.relative_to(self.config.content_dir)))
            for file in files
            if file.is_file()
        ]

    def get_text_document_preview(self, path: str):
        file_path = self.config.content_dir / path
        with open(file_path, "r") as f:
            content = f.read(100)
        return content

    def get_sibling_content_folders(self, path: str = ""):
        search_contnet_path = self.config.content_dir / path
        files = search_contnet_path.glob("*")
        return [
            (file.name, str(file.relative_to(self.config.content_dir)))
            for file in files
            if file.is_dir()
        ]
